//M10 thread parameters
OD=10;
P=1.5;
L=15;
TD=0.92;

$fa=1; $fs=0.3;    

/**** module polyline(points ...) *****
* plots a 2D line linking a list of coordinates 
* Req. Parameters:
* points - list of [x,y] coordinates, must not be empty, the line can degenerate to a single point
* Opt. Parameters:
* lw - line width, default 0.1
*/
module polyline(points, lw=0.1) {
    if(len(points)>1){
        for (i= [1 : len(points)-1 ]) hull(){
          translate(points[i-1]) circle(d=lw, $fn=10);
          translate(points[i]) circle(d=lw, $fn=10);
        }
    }else{
        translate(points[1]) circle(d=lw, $fn=10);
    }
}

/**** function arc2l(r ...) *****
* Generates list of 2D points for a simple or spiral arc
* Req. Parameters:
* r - radius
* Opt. Parameters:
* center - [x,y] coordinate of center, default [0,0]
* ang - [start,end] angle to encompass, default full circle
* ri - spiral radius increment per full turn, default 0 (no spiral)
* $fn,$fa,$fs - as usual per full circle, except the number of segments will be rounded up  
*/
function arc2l(r, center=[0,0], ang=[0,360], ri=0, $fn=$fn, $fa=$fa, $fs=$fs)
    =let( da=ang[1]-ang[0], ns=ceil(abs(da/360)*($fn>0?($fn>=3?$fn:3):ceil(max(min(360/$fa,r*2*PI/$fs),5)))) )
    [for(a=[ang[0]:da/ns:ang[1]])[(r+ri*(a-ang[0])/360)*cos(a)+center.x, (r+ri*(a-ang[0])/360)*sin(a)+center.y]];

/**** example *****/   
    
//Generate the thread cross section (XY plane) 
shape2=concat(arc2l(OD/2,ang=[0,22.5]),arc2l(OD/2,ang=[22.5,135],ri=-3.2*TD),arc2l(OD/2-TD,ang=[135,225]),arc2l(OD/2-TD,ang=[225,337.5],ri=3.2*TD),arc2l(OD/2,ang=[337.5,360]));  

//!linear_extrude(0.1) polyline(shape2); //Debug cross section

linear_extrude(height = L, twist = -360*L/P)
polygon(shape2);  //Do the threaded rod

